@@ -4,7 +4,7 @@
 # Tests for the `humanfriendly' package.
 #
 # Author: Peter Odding <peter.odding@paylogic.eu>
-# Last Change: July 16, 2017
+# Last Change: January 13, 2018
 # URL: https://humanfriendly.readthedocs.io
 
 """Test suite for the `humanfriendly` package."""
@@ -52,8 +52,11 @@ from humanfriendly.terminal import (
     connected_to_terminal,
     find_terminal_size,
     get_pager_command,
+    message,
+    output,
     show_pager,
     terminal_supports_colors,
+    warning,
 )
 from humanfriendly.testing import (
     CallableTimedOut,
@@ -313,11 +316,17 @@ class HumanFriendlyTestCase(TestCase):
         self.assertEqual('1 GB', humanfriendly.format_size(1000 ** 3))
         self.assertEqual('1 TB', humanfriendly.format_size(1000 ** 4))
         self.assertEqual('1 PB', humanfriendly.format_size(1000 ** 5))
+        self.assertEqual('1 EB', humanfriendly.format_size(1000 ** 6))
+        self.assertEqual('1 ZB', humanfriendly.format_size(1000 ** 7))
+        self.assertEqual('1 YB', humanfriendly.format_size(1000 ** 8))
         self.assertEqual('1 KiB', humanfriendly.format_size(1024 ** 1, binary=True))
         self.assertEqual('1 MiB', humanfriendly.format_size(1024 ** 2, binary=True))
         self.assertEqual('1 GiB', humanfriendly.format_size(1024 ** 3, binary=True))
         self.assertEqual('1 TiB', humanfriendly.format_size(1024 ** 4, binary=True))
         self.assertEqual('1 PiB', humanfriendly.format_size(1024 ** 5, binary=True))
+        self.assertEqual('1 EiB', humanfriendly.format_size(1024 ** 6, binary=True))
+        self.assertEqual('1 ZiB', humanfriendly.format_size(1024 ** 7, binary=True))
+        self.assertEqual('1 YiB', humanfriendly.format_size(1024 ** 8, binary=True))
         self.assertEqual('45 KB', humanfriendly.format_size(1000 * 45))
         self.assertEqual('2.9 TB', humanfriendly.format_size(1000 ** 4 * 2.9))
 
@@ -333,8 +342,14 @@ class HumanFriendlyTestCase(TestCase):
         self.assertEqual(1024, humanfriendly.parse_size('1 kilobyte', binary=True))
         self.assertEqual(1000 ** 2 * 69, humanfriendly.parse_size('69 MB'))
         self.assertEqual(1000 ** 3, humanfriendly.parse_size('1 GB'))
+        self.assertEqual(1000 ** 4, humanfriendly.parse_size('1 TB'))
+        self.assertEqual(1000 ** 5, humanfriendly.parse_size('1 PB'))
+        self.assertEqual(1000 ** 6, humanfriendly.parse_size('1 EB'))
+        self.assertEqual(1000 ** 7, humanfriendly.parse_size('1 ZB'))
+        self.assertEqual(1000 ** 8, humanfriendly.parse_size('1 YB'))
         self.assertEqual(1000 ** 3 * 1.5, humanfriendly.parse_size('1.5 GB'))
-        self.assertRaises(humanfriendly.InvalidSize, humanfriendly.parse_size, '1z')
+        self.assertEqual(1024 ** 8 * 1.5, humanfriendly.parse_size('1.5 YiB'))
+        self.assertRaises(humanfriendly.InvalidSize, humanfriendly.parse_size, '1q')
         self.assertRaises(humanfriendly.InvalidSize, humanfriendly.parse_size, 'a')
 
     def test_format_length(self):
@@ -527,19 +542,17 @@ class HumanFriendlyTestCase(TestCase):
         # Test automatic timer.
         automatic_timer = humanfriendly.Timer()
         time.sleep(1)
-        self.assertEqual(normalize_timestamp(humanfriendly.round_number(
-            automatic_timer.elapsed_time,
-            keep_width=True,
-        )), '1.00')
+        # XXX The following normalize_timestamp(ndigits=0) calls are intended
+        #     to compensate for unreliable clock sources in virtual machines
+        #     like those encountered on Travis CI, see also:
+        #     https://travis-ci.org/xolox/python-humanfriendly/jobs/323944263
+        self.assertEqual(normalize_timestamp(automatic_timer.elapsed_time, 0), '1.00')
         # Test resumable timer.
         resumable_timer = humanfriendly.Timer(resumable=True)
         for i in range(2):
             with resumable_timer:
                 time.sleep(1)
-        self.assertEqual(normalize_timestamp(humanfriendly.round_number(
-            resumable_timer.elapsed_time,
-            keep_width=True,
-        )), '2.00')
+        self.assertEqual(normalize_timestamp(resumable_timer.elapsed_time, 0), '2.00')
         # Make sure Timer.__enter__() returns the timer object.
         with humanfriendly.Timer(resumable=True) as timer:
             assert timer is not None
@@ -672,6 +685,17 @@ class HumanFriendlyTestCase(TestCase):
         random_byte_count = random.randint(1024, 1024 * 1024)
         returncode, output = run_cli(main, '--format-size=%i' % random_byte_count)
         assert output.strip() == humanfriendly.format_size(random_byte_count)
+        # Test `humanfriendly --format-size --binary'.
+        random_byte_count = random.randint(1024, 1024 * 1024)
+        returncode, output = run_cli(main, '--format-size=%i' % random_byte_count, '--binary')
+        assert output.strip() == humanfriendly.format_size(random_byte_count, binary=True)
+        # Test `humanfriendly --format-length'.
+        random_len = random.randint(1024, 1024 * 1024)
+        returncode, output = run_cli(main, '--format-length=%i' % random_len)
+        assert output.strip() == humanfriendly.format_length(random_len)
+        random_len = float(random_len) / 12345.6
+        returncode, output = run_cli(main, '--format-length=%f' % random_len)
+        assert output.strip() == humanfriendly.format_length(random_len)
         # Test `humanfriendly --format-table'.
         returncode, output = run_cli(main, '--format-table', '--delimiter=\t', input='1\t2\t3\n4\t5\t6\n7\t8\t9')
         assert output.strip() == dedent('''
@@ -688,6 +712,14 @@ class HumanFriendlyTestCase(TestCase):
         # Test `humanfriendly --parse-size'.
         returncode, output = run_cli(main, '--parse-size=5 KB')
         assert int(output) == humanfriendly.parse_size('5 KB')
+        # Test `humanfriendly --parse-size'.
+        returncode, output = run_cli(main, '--parse-size=5 YiB')
+        assert int(output) == humanfriendly.parse_size('5 YB', binary=True)
+        # Test `humanfriendly --parse-length'.
+        returncode, output = run_cli(main, '--parse-length=5 km')
+        assert int(output) == humanfriendly.parse_length('5 km')
+        returncode, output = run_cli(main, '--parse-length=1.05 km')
+        assert float(output) == humanfriendly.parse_length('1.05 km')
         # Test `humanfriendly --run-command'.
         returncode, output = run_cli(main, '--run-command', 'bash', '-c', 'sleep 2 && exit 42')
         assert returncode == 42
@@ -700,6 +732,10 @@ class HumanFriendlyTestCase(TestCase):
         assert ansi_style(inverse=True) == '%s7%s' % (ANSI_CSI, ANSI_SGR)
         assert ansi_style(strike_through=True) == '%s9%s' % (ANSI_CSI, ANSI_SGR)
         assert ansi_style(color='blue') == '%s34%s' % (ANSI_CSI, ANSI_SGR)
+        assert ansi_style(background='blue') == '%s44%s' % (ANSI_CSI, ANSI_SGR)
+        assert ansi_style(color='blue', bright=True) == '%s94%s' % (ANSI_CSI, ANSI_SGR)
+        assert ansi_style(color=214) == '%s38;5;214%s' % (ANSI_CSI, ANSI_SGR)
+        assert ansi_style(background=214) == '%s39;5;214%s' % (ANSI_CSI, ANSI_SGR)
         self.assertRaises(ValueError, ansi_style, color='unknown')
 
     def test_ansi_width(self):
@@ -726,6 +762,41 @@ class HumanFriendlyTestCase(TestCase):
         # Make sure ansi_wrap() ends the text by resetting the ANSI styles.
         assert ansi_wrap(text, bold=True).endswith(ANSI_RESET)
 
+    def test_generate_output(self):
+        """Test the :func:`humanfriendly.terminal.output()` function."""
+        text = "Standard output generated by output()"
+        with CaptureOutput(merged=False) as capturer:
+            output(text)
+            self.assertEqual([text], capturer.stdout.get_lines())
+            self.assertEqual([], self.ignore_coverage_warning(capturer))
+
+    def test_generate_message(self):
+        """Test the :func:`humanfriendly.terminal.message()` function."""
+        text = "Standard error generated by message()"
+        with CaptureOutput(merged=False) as capturer:
+            message(text)
+            self.assertEqual([], capturer.stdout.get_lines())
+            self.assertIn(text, self.ignore_coverage_warning(capturer))
+
+    def test_generate_warning(self):
+        """Test the output(), message() and warning() functions."""
+        text = "Standard error generated by warning()"
+        with CaptureOutput(merged=False) as capturer:
+            warning(text)
+            self.assertEqual([], capturer.stdout.get_lines())
+            self.assertEqual([ansi_wrap(text, color='red')], self.ignore_coverage_warning(capturer))
+
+    def ignore_coverage_warning(self, capturer):
+        """
+        Filter out coverage.py warning from standard error.
+
+        This is intended to remove the following line from the lines captured
+        on the standard error stream:
+
+        Coverage.py warning: No data was collected. (no-data-collected)
+        """
+        return [line for line in capturer.stderr.get_lines() if 'no-data-collected' not in line]
+
     def test_clean_output(self):
         """Test :func:`humanfriendly.terminal.clean_terminal_output()`."""
         # Simple output should pass through unharmed (single line).
